using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Threading;

namespace YAPS
{
    /*
     * YAPS - Yet Another Proxy Server: UDP Multicast to TCP Unicast Proxy (with VCR)
     * (C) Daniel Kirstenpfad 2006, HTTP portions (C) 2001 by Sam Pullara; ICON (C) 2003-2004 by Steven W. Smith
     * 
     * This is a small tool which allows you to proxy udp rtp multicast to tcp unicast (wihtout the rtp enclosure).
     * Beside this functionality YAPS implements a HTTP webserver and a VCR.
     * 
     * For updates and more information visit the YAPS website: http://www.technology-ninja.com
     * 
     */
    // TODO: (NEW!) sign for the RecordingList
    // TODO: implement repetitive events
    // TODO: Settings Page + Exportables
    // TODO: add feature to delete a recording without knowing it's guid
    // TODO: unify the /vcr/ path to something that determines the called functionality by a parameter like /vcr/managerecordings?type=delete or ?type=add
    // TODO: add feature that auto-deletes old recordings after a specific number of recordings of that category is reached...
    // TODO: add feature to reset the LastTimePlayed of a Recording...

    /// <summary>
    /// Yet Another Proxy Server: UDP Multicast to TCP Unicast Proxy and VCR
    /// </summary>
    class YAPSApp
	{
		[STAThread]
		static void Main(String[] args)
        {
            #region Logo
            ConsoleOutputLogger.WriteLine("Yet Another Proxy Server: UDP Multicast to TCP Unicast Proxy "+Environment.Version.ToString());
            ConsoleOutputLogger.WriteLine("(C) 2006 Daniel Kirstenpfad - http://www.technology-ninja.com");
            #endregion

            #region Data
            HttpServer httpServer;
            VCRScheduler vcrScheduler;
			int port = 80;
			string docRoot = ".";
            httpServer = new HttpServer(docRoot, port);
            vcrScheduler = new VCRScheduler(httpServer);
            #endregion

            #region Load the Settings...
            SettingsProcessor Configuration = new SettingsProcessor(".\\settings\\YAPS.Recordings.dat", ".\\settings\\YAPS.DoneRecordings.dat", ".\\settings\\YAPS.Categories.dat", ".\\settings\\YAPS.ChannelNameMapping.dat", ".\\settings\\YAPS.ChannelPictureMapping.dat", ".\\settings\\YAPS.Settings.dat", vcrScheduler, httpServer);

            Configuration.LoadSettings();

            // tell the processors that we exist...
            vcrScheduler.Configuration = Configuration;
            httpServer.Configuration = Configuration;
            #endregion

            #region Testing


            List<Recording> blah = Sorter.SortRecordingTable(vcrScheduler.doneRecordings, true);
            ConsoleOutputLogger.WriteLine("Blah");

            // set up testing recording data
            /*Recording testRecording = new Recording();

            testRecording.Channel = "13";
            testRecording.StartsAt = DateTime.Now.AddHours(1);
            testRecording.EndsAt = testRecording.StartsAt.AddMinutes(30);

            testRecording.Recording_Filename = testRecording.Recording_ID.ToString();
            testRecording.Recording_Name = "Das perfekte Dinner";

            vcrScheduler.Recordings.Add(testRecording.Recording_ID, testRecording);
            
            testRecording = new Recording();

            testRecording.Channel = "20";
            testRecording.StartsAt = DateTime.Now.AddDays(1);
            testRecording.EndsAt = testRecording.StartsAt.AddMinutes(30);

            testRecording.Recording_Filename = testRecording.Recording_ID.ToString();
            testRecording.Recording_Name = "testaufnahme";

            vcrScheduler.Recordings.Add(testRecording.Recording_ID, testRecording);
            */
            #endregion

           /* #region channelmappinginit
            // quick-n-dirty hack to get some data into the nameMapper
            MapNames.Add(5, "ZDFtheaterkanal","/images/channels/zdftheater.png");
            MapNames.Add(6, "3Sat", "/images/channels/3sat.png");
            MapNames.Add(9, "KIKA", "/images/channels/kika.png");
            MapNames.Add(10, "ZDF", "/images/channels/zdf.png");
            MapNames.Add(11, "ZDFdokukanal", "/images/channels/zdfdoku.png");
            MapNames.Add(12, "ZDFinfokanal", "/images/channels/zdfinfo.png");
            MapNames.Add(13, "Sat.1", "/images/channels/sat1.png");
            MapNames.Add(14, "Pro7", "/images/channels/pro7.png");
            MapNames.Add(15, "Kabel 1", "/images/channels/kabel1.png");
            MapNames.Add(16, "DSF", "/images/channels/dsf.png");
            MapNames.Add(17, "N24", "/images/channels/n24.png");
            MapNames.Add(18, "RTL", "/images/channels/rtl.png");
            MapNames.Add(19, "RTL.2", "/images/channels/rtl2.png");
            MapNames.Add(20, "VOX", "/images/channels/vox.png");
            MapNames.Add(21, "n-tv", "/images/channels/NTV.png");
            MapNames.Add(22, "SuperRTL", "/images/channels/superrtl.png");
            MapNames.Add(23, "ARD", "/images/channels/ard.png");
            MapNames.Add(24, "arte", "/images/channels/arte.png");
            MapNames.Add(25, "BR", "/images/channels/br.png");
            MapNames.Add(26, "BR-alpha", "/images/channels/br-alpha.png");
            MapNames.Add(27, "HR", "/images/channels/hr.png");
            MapNames.Add(28, "Phoenix", "/images/channels/phoenix.png");
            MapNames.Add(29, "SWR BW", "/images/channels/swr.png");
            MapNames.Add(30, "WDR", "/images/channels/wdr.png");
            MapNames.Add(31, "T5", "/images/channels/tele5.png");
            MapNames.Add(32, "9Live", "");
            MapNames.Add(33, "Traumpartner.TV", "");
            Configuration.SaveSettings();
            #endregion*/
       

            #region Starting up...
            #region HTTP Server
            ConsoleOutputLogger.WriteLine("Starting HTTP Server...");
			    Thread http_server_thread = new Thread(new ThreadStart(httpServer.listen));
			    http_server_thread.Start();
                #endregion

                #region VCR
                ConsoleOutputLogger.WriteLine("Starting VCR Scheduler...");
                Thread vcr_scheduler_thread = new Thread(new ThreadStart(vcrScheduler.RecordingScheduler));
                vcr_scheduler_thread.Start();
                #endregion
            #endregion
        }
	}
}
