using System;
using System.Collections;
using System.Text;

namespace YAPS
{
    /// <summary>
    /// Map the Channelnames to Numbers and vice versa
    /// </summary>
    public static class MapNames
    {
        private static Hashtable ChannelNames = new Hashtable();
        private static Hashtable ChannelPictures = new Hashtable();

        /// <summary>
        /// re-initiates the ChannelNames Hashtable; DO ONLY USE THIS WHEN LOADING SETTINGS!!!
        /// </summary>
        /// <param name="LoadedNameMapping"></param>
        public static void SetInitialNameMapping(Hashtable LoadedNameMapping)
        {
            // TODO: check for possible errors....
            ChannelNames = LoadedNameMapping;
        }

        /// <summary>
        /// re-initiates the ChannelPictures Hashtable; DO ONLY USE THIS WHEN LOADING SETTINGS!!!
        /// </summary>
        /// <param name="LoadedNameMapping"></param>
        public static void SetInitialPictureMapping(Hashtable LoadedPictureMapping)
        {
            // TODO: check for possible errors....
            ChannelPictures = LoadedPictureMapping;
        }


        #region Add/Remove

        #region Add
        /// <summary>
        /// Adds a Number+Name Combination to the Mapping table
        /// </summary>
        /// <param name="Number">Channel-Number</param>
        /// <param name="Name">Channel-Name</param>
        /// <returns>true if successfully added, false if already existed/failed</returns>
        public static bool Add(String Number, String Name, String Picture)
        {
            try
            {
                if (!ChannelNames.ContainsKey(Convert.ToInt32(Number)))
                {
                    ChannelNames.Add(Convert.ToInt32(Number), Name);
                    if (!ChannelPictures.ContainsKey(Name))
                    {
                        ChannelPictures.Add(Name, Picture);
                    }
                    else
                        return false;
                    return true;
                }
                else 
                    return false;
            }
            catch(Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Add: " + e.Message);
                return false;
            }
        }
        /// <summary>
        /// Adds a Number+Name Combination to the Mapping table
        /// </summary>
        /// <param name="Number">Channel-Number</param>
        /// <param name="Name">Channel-Name</param>
        /// <returns>true if successfully added, false if already existed/failed</returns>
        public static bool Add(Int32 Number, String Name, String Picture)
        {
            return Add(Convert.ToString(Number), Name, Picture);
        }
        #endregion

        #region Remove
        /// <summary>
        /// Remove a Name Mapping from the Mapping table
        /// </summary>
        /// <param name="Number">the Channel Number</param>
        /// <returns>true if successful, false if not</returns>
        public static bool Remove(Int32 Number)
        {
            try
            {
                ChannelPictures.Remove(Number2Name(Number));
                ChannelNames.Remove(Number);
                return true;
            }
            catch (Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Remove: " + e.Message);
                return false;
            }
        }
        /// <summary>
        /// Remove a Name Mapping from the Mapping table
        /// </summary>
        /// <param name="Name">the Channel Name</param>
        /// <returns>true if successful, false if not</returns>
        public static bool Remove(String Name)
        {
            try
            {
                return Remove(Name2Number(Name));
            }
            catch (Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Remove: " + e.Message);
                return false;
            }
        }

        #endregion

        #region Edit
        public static bool Edit(String OriginalChannelName,int ChannelNumber,String NewChannelName,String NewChannelPicture)
        {
            try
            {
                // first remove the entry
                MapNames.Remove(OriginalChannelName);
                // then add it again
                MapNames.Add(ChannelNumber, NewChannelName, NewChannelPicture);
                return true;
            }
            catch (Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Edit: " + e.Message);
                return false;
            }
        }
        public static bool Edit(String OriginalChannelName, String ChannelNumber, String NewChannelName, String NewChannelPicture)
        {
            return Edit(OriginalChannelName, Convert.ToInt32(ChannelNumber), NewChannelName, NewChannelPicture);
        }
        #endregion

        #endregion

        #region Mapping
        public static String Number2Name(int ChannelNumber)
        {
            return (String)ChannelNames[ChannelNumber];
        }
        public static Int32 Name2Number(String ChannelName)
        {
           foreach (Int32 Key in ChannelNames.Keys)
           {
             if (((string)ChannelNames[Key]).ToUpper() == ChannelName.ToUpper())
             {
                return (Int32)Key;
             }
           }
           return -1;
        }
        public static String Name2Picture(String ChannelName)
        {
            foreach (String Name in ChannelPictures.Keys)
            {
                if (((string)Name).ToUpper() == ChannelName.ToUpper())
                {
                    return (string)ChannelPictures[Name];
                }
            }
            return "";
        }
        public static String Number2Picture(int ChannelNumber)
        {
            return Name2Picture((String)ChannelNames[ChannelNumber]);
        }
        #endregion

        #region Listings
        /// <summary>
        /// Returns the Mapping Table
        /// </summary>
        /// <returns></returns>
        public static Hashtable GetNamesMapTable()
        {
            return ChannelNames;
        }

        public static Hashtable GetPicturesMapTable()
        {
            return ChannelPictures;
        }
        public static bool HasPicture(String ChannelName)
        {
            if (Name2Picture(ChannelName) != "")
                return true;
            else
                return false;
        }
        public static bool HasPicture(Int32 ChannelNumber)
        {
            return HasPicture(Number2Name(ChannelNumber));
        }
        #endregion
    }
}
