using System;
using System.Collections.Generic;
using System.Text;

namespace YAPS
{
    /// <summary>
    /// This Class stores a number of Console Output Lines into a ring buffer
    /// </summary>
    public static class ConsoleOutputLogger
    {
        private static int Max_Number_Of_Entries = 200;
        private static LinkedList<String> LoggerList = new LinkedList<String>();

        public static void SetNumberOfMaxEntries(int Number)
        {
            // TODO: It would be nice to keep at least the Number of Lines we're setting
            lock(LoggerList)
            {
                LoggerList.Clear();
            }
            Max_Number_Of_Entries = Number;
        }

        public static int GetMaxNumberOfEntries()
        {
            return Max_Number_Of_Entries;
        }

        public static void WriteLine(String text)
        {
            DateTime TimeDate = DateTime.Now;

            text = TimeDate.ToShortDateString() + " - " + TimeDate.ToShortTimeString() + " " + text;
            // write it to the console
            Console.WriteLine(text);

            lock(LoggerList)
            {
                if (LoggerList.Count == Max_Number_Of_Entries)
                {
                    LoggerList.RemoveFirst();
                }
                
                LoggerList.AddLast(text);
            }
        }

        public static String[] GetLoggedLines()
        {
            String[] Output = new String[Max_Number_Of_Entries];
            int Current_Position = 0;

            foreach (String line in LoggerList)
            {
                Output[Current_Position] = line;
                Current_Position++;
            }
            return Output;
        }
    }
}
