using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace YAPS
{
    public class SettingsProcessor
    {
        private String RecordingsFilename;
        private String DoneRecordingsFilename;
        private String CategoriesFilename;
        private String ChannelNameMappingFilename;
        private String MiscSettingsFilename;
        private String ChannelPictureMappingFilename;
        private VCRScheduler internal_vcr_scheduler;
        private HttpServer internal_http_server;

        // TODO: redo the Variable names...
        /// <summary>
        /// Creates the Settings Processor
        /// </summary>
        /// <param name="RFilename">The Recording-Settings-Path+Filename</param>
        /// <param name="DRFilename">The doneRecording-Settings-Path+Filename</param>
        /// <param name="CFilename">The Categories-Settings-Path+Filename</param>
        /// <param name="CMFilename">The ChannelNameMapping-Settings-Path+Filename</param>
        /// <param name="CPFilename">The ChannelPictureMapping-Settings-Path+Filename</param>
        /// <param name="SeFilename">The Misc-Settings-Path+Filename</param>
        /// <param name="vcrScheduler">object of the vcrScheduler</param>
        /// <param name="webserver">object of the webserver</param>
        public SettingsProcessor(String RFilename, String DRFilename,String CFilename, String CMFilename, String CPFilename, String SeFilename,VCRScheduler vcrScheduler, HttpServer webserver)
        {
            RecordingsFilename = RFilename;
            DoneRecordingsFilename = DRFilename;
            CategoriesFilename = CFilename;
            ChannelNameMappingFilename = CMFilename;
            MiscSettingsFilename = SeFilename;
            ChannelPictureMappingFilename = CPFilename;
            internal_http_server = webserver;
            internal_vcr_scheduler = vcrScheduler;
        }

        #region SaveSettings
        /// <summary>
        /// this method saves the settings to the actual config file
        /// </summary>
        public void SaveSettings()
        {
            FileStream fs;
            // TODO: make it thread safe

            ConsoleOutputLogger.WriteLine("Saving Settings...");

            #region Categories
            fs = new FileStream(CategoriesFilename, FileMode.OpenOrCreate, FileAccess.Write);
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                bf.Serialize(fs, internal_vcr_scheduler.Categories);
            }
            finally
            {
                fs.Close();
            }
            #endregion

            #region Channel-Number-Name-Mapping
            fs = new FileStream(ChannelNameMappingFilename, FileMode.OpenOrCreate, FileAccess.Write);
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                bf.Serialize(fs, MapNames.GetNamesMapTable());
            }
            finally
            {
                fs.Close();
            }
            #endregion

            #region Channel-Name-Picture-Mapping
            fs = new FileStream(ChannelPictureMappingFilename, FileMode.OpenOrCreate, FileAccess.Write);
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                bf.Serialize(fs, MapNames.GetPicturesMapTable());
            }
            finally
            {
                fs.Close();
            }
            #endregion

            #region Recordings
            fs = new FileStream (RecordingsFilename,FileMode.OpenOrCreate ,FileAccess.Write );
            try
            {
                BinaryFormatter bf=new BinaryFormatter ();
                bf.Serialize(fs, internal_vcr_scheduler.Recordings);
            }
            finally
            {
                fs.Close ();
            }
            #endregion

            #region DoneRecordings
            fs = new FileStream(DoneRecordingsFilename, FileMode.OpenOrCreate, FileAccess.Write);
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                bf.Serialize(fs, internal_vcr_scheduler.doneRecordings);
            }
            finally
            {
                fs.Close();
            }
            #endregion

/*            #region Misc Settings
            fs = new FileStream(MiscSettingsFilename, FileMode.OpenOrCreate, FileAccess.Write);
            try
            {
                BinaryFormatter bf = new BinaryFormatter();
                bf.Serialize(fs,Settings);
            }
            finally
            {
                fs.Close();
            }
            #endregion*/

        }
        #endregion

        #region LoadSettings
        /// <summary>
        /// guess what it does...it Loads the Settings from the Config File; it's usually only called at startup time
        /// </summary>
        public void LoadSettings()
        {
            // TODO: Error checking; what happens when the config files are corrupt?

            FileStream fs;

            ConsoleOutputLogger.WriteLine("Loading Settings...");

            #region Categories
            if (File.Exists(CategoriesFilename))
            {
                fs = new FileStream(CategoriesFilename, FileMode.Open, FileAccess.Read);
                try
                {
                    BinaryFormatter bf = new BinaryFormatter();
                    internal_vcr_scheduler.Categories = (List<Category>)bf.Deserialize(fs);
                }
                finally
                {
                    fs.Close();
                }
            }
            else
                ConsoleOutputLogger.WriteLine("No Categories configfile found.");
            #endregion

            #region ChannelNameMapping
            if (File.Exists(ChannelNameMappingFilename))
            {
                fs = new FileStream(ChannelNameMappingFilename, FileMode.Open, FileAccess.Read);
                try
                {
                    BinaryFormatter bf = new BinaryFormatter();
                    MapNames.SetInitialNameMapping((Hashtable)bf.Deserialize(fs));
                }
                finally
                {
                    fs.Close();
                }
            }
            else
                ConsoleOutputLogger.WriteLine("No ChannelNameMapping configfile found.");
            #endregion

            #region ChannelPictureMapping
            if (File.Exists(ChannelPictureMappingFilename))
            {
                fs = new FileStream(ChannelPictureMappingFilename, FileMode.Open, FileAccess.Read);
                try
                {
                    BinaryFormatter bf = new BinaryFormatter();
                    MapNames.SetInitialPictureMapping((Hashtable)bf.Deserialize(fs));
                }
                finally
                {
                    fs.Close();
                }
            }
            else
                ConsoleOutputLogger.WriteLine("No ChannelPictureMapping configfile found.");
            #endregion


            #region Recordings
            if (File.Exists(RecordingsFilename))
            {
                fs = new FileStream(RecordingsFilename, FileMode.Open, FileAccess.Read);
                try
                {
                    BinaryFormatter bf = new BinaryFormatter();
                    internal_vcr_scheduler.Recordings = (Hashtable)bf.Deserialize(fs);
                }
                finally
                {
                    fs.Close();
                }
            }
            else
                ConsoleOutputLogger.WriteLine("No Recordings configfile found.");
            #endregion

            #region DoneRecordings
            if (File.Exists(DoneRecordingsFilename))
            {
                fs = new FileStream(DoneRecordingsFilename, FileMode.Open, FileAccess.Read);
                try
                {
                    BinaryFormatter bf = new BinaryFormatter();
                    internal_vcr_scheduler.doneRecordings = (Hashtable)bf.Deserialize(fs);
                }
                finally
                {
                    fs.Close();
                }
            }
            else
                ConsoleOutputLogger.WriteLine("No DoneRecordings configfile found.");
            #endregion

 /*           #region MiscSettings
            if (File.Exists(MiscSettingsFilename))
            {
                fs = new FileStream(MiscSettingsFilename, FileMode.Open, FileAccess.Read);
                try
                {
                    BinaryFormatter bf = new BinaryFormatter();
                    Settings.SetInitialSettings((SettingsData)bf.Deserialize(fs));
                }
                finally
                {
                    fs.Close();
                }
            }
            else
                ConsoleOutputLogger.WriteLine("No Misc-Settings configfile found.");
            #endregion*/
        }
        #endregion
    }
}