using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

namespace YAPS
{
    /// <summary>
    /// each file request that leads to a .html file is passed through here
    /// </summary>
    public class TemplateProcessor
    {
        #region internal Data
        private VCRScheduler internal_vcrscheduler;
        //private HttpProcessor internal_httpprocessor;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor of the Templace Processor
        /// </summary>
        /// <param name="vcrscheduler">the VCRScheduler...so we can get the </param>
        public TemplateProcessor(VCRScheduler vcrscheduler)
        {
            internal_vcrscheduler = vcrscheduler;
        }
        #endregion

        #region ProcessHTMLTemplate
        /// <summary>
        /// Processes the actual HTML Sourcecode
        /// </summary>
        /// <param name="template_filename"></param>
        /// <returns>the actual HTML code after the template parsing and keyword replaceing</returns>
        public String ProcessHTMLTemplate(String template_filename,String Querystring)
        {
            String Output_HTML_Code = "";
            DateTime Started = DateTime.Now;

            #region read the template file
            using (StreamReader sr = File.OpenText(template_filename))
            {
                String buffer;

                while ((buffer = sr.ReadLine()) != null)
                {
                    Output_HTML_Code = Output_HTML_Code + buffer;
                    // read...
                }
                sr.Close();
            }
            #endregion

            #region replace the placeholders...

            // TODO: at the moment the placeholders must be lower case only to be detected - change that!

            #region %recording_listing%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%recording_listing%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%recording_listing%", Template_Recording_Listing());
            }
            #endregion

            #region %querystring%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%querystring%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%querystring%", Querystring);
            }
            #endregion

            #region %recorded_listing%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%recorded_listing%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%recorded_listing%", Template_Recorded_Listing());
            }
            #endregion

            #region %console_output%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%console_output%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%console_output%", Template_Console_Output());
            }
            #endregion

            #region %space_usage_bar%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%space_usage_bar%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%space_usage_bar%", Template_Space_Usage_Bar());
            }
            #endregion

            #region %page_render_time%
            DateTime ended = DateTime.Now;
            while (Output_HTML_Code.Contains("%page_render_time%"))
            {
                TimeSpan render_time = new TimeSpan(ended.Ticks-Started.Ticks);

                Output_HTML_Code = Output_HTML_Code.Replace("%page_render_time%", Convert.ToString(render_time.TotalMilliseconds)+"msecs");
            }

            #endregion

            #endregion

            return Output_HTML_Code;
        }
        #endregion

        #region HTML Code Generators

        #region Space_Usage_Bar
        /// <summary>
        /// Draws a Percentage Bar of the ration of free and used space
        /// </summary>
        /// <returns>HTML Code</returns>
        String Template_Space_Usage_Bar()
        {
        	StringBuilder Output = new StringBuilder();
        	long FreeSpace = 1;
        	long TotalSize = 1;
        	
        	#if MONO
        	int longestMatch = 0;
        	Mono.Unix.UnixDriveInfo[] mountPoints = Mono.Unix.UnixDriveInfo.GetDrives();
        	foreach(Mono.Unix.UnixDriveInfo mountPoint in mountPoints) //we get only the mountpoints, so search for the mountpoint, which contains this directory
        	{
        		//search for the mount point which has the longest match (do not take "/" as mountpoint for "/mnt/data/Video" but "/mnt/data")
        		if (Environment.CurrentDirectory.StartsWith(mountPoint.RootDirectory.FullName) && longestMatch < mountPoint.RootDirectory.FullName.Length)
        		{
        			FreeSpace = mountPoint.AvailableFreeSpace;
        			TotalSize = mountPoint.TotalSize;
        			longestMatch = mountPoint.RootDirectory.FullName.Length;
        		}
        	}
        	#else
        	  // TODO: configurable recording storeing path...
            System.IO.DriveInfo DiskInfo = new DriveInfo(Environment.CurrentDirectory);

            FreeSpace = DiskInfo.AvailableFreeSpace;
            TotalSize = DiskInfo.TotalSize;
        	
        	#endif
            float fFreePercent = FreeSpace/((TotalSize)/100);
            
            int FreePercent = Convert.ToInt32(fFreePercent);
            int UsedPercent = 100-FreePercent;


            // HACK: since I don't want to get every filesize at this moment I am estimating on base of the used space
            #region the blue bar
            // Calculate how much minutes used all our recordings are
            TimeSpan RecordedTime = new TimeSpan(0);
            lock (internal_vcrscheduler.doneRecordings)
            {
                foreach (Recording recording_entry in internal_vcrscheduler.doneRecordings.Values)
                {
                    if (!recording_entry.CurrentlyRecording)
                        RecordedTime = RecordedTime.Add(new TimeSpan((recording_entry.EndsAt.Ticks - recording_entry.StartsAt.Ticks)));
                }
            }
            
            // Calculate how much minutes will be used by the scheduled recordings            
            TimeSpan ToBeRecordedTime = new TimeSpan(0);

            lock (internal_vcrscheduler.Recordings)
            {
                foreach (Recording recording_entry in internal_vcrscheduler.Recordings.Values)
                {
                        ToBeRecordedTime = ToBeRecordedTime.Add(new TimeSpan((recording_entry.EndsAt.Ticks - recording_entry.StartsAt.Ticks)));
                }
            }

            // we estimate now how many percent of the free space would be used after all scheduled recordings are done

            // calculate the average Bitrate per Second
            double AvgBitrate = Convert.ToInt32((TotalSize - FreeSpace) / RecordedTime.TotalSeconds);
            
            // calculate the needed Space for all our recordings
            double EstimatedNeededSpace = (AvgBitrate * ToBeRecordedTime.TotalSeconds);

            // how much percent of the free space is this?
            int EstimatedNeededSpacePercent = Convert.ToInt32((EstimatedNeededSpace / ((FreeSpace) / 100)));

            if (EstimatedNeededSpacePercent >= FreePercent)
            {
                // when more space is needed than available, draw the normal bar with no green part and the exclamation mark
                Output.Append("<table boder=\"0\" CellSpacing=\"0\" CellPadding=\"0\" width=\"50%\"><tr>");
                Output.Append("<td style=\"background-image: url(images/percent_bar_tile_red.png); background-repeat: repeat-x; width: ");
                Output.Append(UsedPercent.ToString());
                Output.Append("%; height: 25px; text-align: center;\"></td>");
                //Output.Append(UsedPercent.ToString() + "%</td>");
                Output.Append("<td style=\"background-image: url(images/percent_bar_tile_blue.png); background-repeat: repeat-x; width: ");
                Output.Append(FreePercent.ToString());
                Output.Append("%; height: 25px; text-align: center;\"></td>");
                Output.Append("<td style=\"height:25px; text-align:center;\"><img src=\"images/exclamationmark.png\" border=\"0\" alt=\"Zuwenig Speicherplatz fr alle Aufnahmen\"></td>");
                Output.Append("</tr></table>");
            }
            else
            {
                // draw red,blue,green
                FreePercent = FreePercent - EstimatedNeededSpacePercent;

                Output.Append("<table boder=\"0\" CellSpacing=\"0\" CellPadding=\"0\" width=\"50%\"><tr>");
                Output.Append("<td style=\"background-image: url(images/percent_bar_tile_red.png); background-repeat: repeat-x; width: ");
                Output.Append(UsedPercent.ToString());
                Output.Append("%; height: 25px; text-align: center;\"></td>");
                Output.Append("<td style=\"background-image: url(images/percent_bar_tile_blue.png); background-repeat: repeat-x; width: ");
                Output.Append(EstimatedNeededSpacePercent.ToString());
                Output.Append("%; height: 25px; text-align: center;\"></td>");
                Output.Append("<td style=\"background-image: url(images/percent_bar_tile_green.png); background-repeat: repeat-x; width: ");
                Output.Append(FreePercent.ToString());
                Output.Append("%; height: 25px; text-align: center;\"></td>");                
                Output.Append("</tr></table>");
            }
            

            #endregion
            return Output.ToString();
        }
        #endregion

        #region RecordingListing
        /// <summary>
        /// this generates the Record Listing HTML Sourcecode
        /// </summary>
        /// <returns>Record Listing Sourcecode</returns>
        String Template_Recording_Listing()
        {
            // TODO: Sort the Recording Listings...
            StringBuilder Output = new StringBuilder();

            // BUG: when there's nothing currently recording the header of the table alone is shown...fix that

            // start the table

            // TODO: add a progress bar to each Status
            Output.Append("<table id=\"TABLE1\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\">	<tr>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 55px;\">            <strong>Status</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); width: 169px; background-repeat: repeat-x; height: 21px; text-align: center\">            <strong>Start</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); width: 171px; background-repeat: repeat-x; height: 21px; text-align: center\">            <strong>Ende</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 106px;\">            <strong>Sender</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 277px;\">            <strong>Name</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 79px;\">            <strong>Kategorie</strong></td>        <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x;            height: 21px; text-align: center; width: 60px;\"></td>	</tr>");
            // OLD Output.Append("<table width=\"90%\"><tr style=\"background-color: #93939a;\"><td style=\"width:50px\" align=\"center\">Status</td><td align=\"center\">Time</td><td align=\"center\">Sender</td><td align=\"center\">Name</td></tr>");

            if (internal_vcrscheduler.doneRecordings.Count > 0)
            {
                List<Recording> sortedDoneRecordingList;
                lock (internal_vcrscheduler.doneRecordings.SyncRoot)
                {
                    // TODO: add ascending/descending setting + reimplement the sorting algorithm
                    sortedDoneRecordingList = Sorter.SortRecordingTable(internal_vcrscheduler.doneRecordings, true);
                }

                foreach (Recording recording_entry in sortedDoneRecordingList)
                    {
                    if (recording_entry.CurrentlyRecording == true)
                    {
                        Output.Append("<tr>            <td style=\"text-align: center; width: 60px;\">            <img src=\"images/recordbutton.png\" border=\"0\" /></td>            <td style=\"text-align: center; width: 169px;\">");

                        // OLD Output.Append("<tr style=\"background-color: #a29ca2;\"><td style=\"background-color:red;width:50px\"></td><td align=\"center\" style=\"width:150px\">");

                        // Start Date and Time
                        Output.Append(recording_entry.StartsAt.ToShortDateString());
                        Output.Append(" ");
                        Output.Append(recording_entry.StartsAt.ToShortTimeString());
                        Output.Append("</td><td style=\"text-align: center; width: 171px;\">");
                        
                        // End Date and Time
                        Output.Append(recording_entry.EndsAt.ToShortDateString());
                        Output.Append(" ");
                        Output.Append(recording_entry.EndsAt.ToShortTimeString());

                        // Channel
                        Output.Append("</td><td style=\"text-align: center; width: 106px;\">  ");
                        // HACK: change that!!
                        // TODO: Channel Picture...
                        try
                        {
                            if (MapNames.HasPicture(Convert.ToInt32(recording_entry.Channel)))
                            {
                                Output.Append("<img src=\"");
                                Output.Append(MapNames.Name2Picture(MapNames.Number2Name(Convert.ToInt32(recording_entry.Channel))));
                                Output.Append("\" border=\"0\">");
                            }
                            else
                            {
                                Output.Append(recording_entry.Channel);
                            }
                        }
                        catch (Exception)
                        {
                            Output.Append(recording_entry.Channel);
                        }


                        // Recording Name
                        Output.Append("</td><td style=\"text-align: center; width: 277px;\">");
                        Output.Append(recording_entry.Recording_Name);
                        Output.Append("</td>");

                        // Categorie
                        // TODO: Add Categories...
                        Output.Append("<td style=\"text-align: center; width: 79px;\">todo</td>");
                        // Buttons
                        // TODO: Add Buttons...
                        Output.Append("<td style=\"text-align: center; width: 60px;\">                <img src=\"images/edit_button.png\" border=\"0\" alt=\"Editieren\" />                <img src=\"images/delete_button.png\" border=\"0\" alt=\"Lschen\" />            </td>");
                    }
                }

            }
            // just work when there is work to do...
            if (internal_vcrscheduler.Recordings.Count > 0)
            {
                List<Recording> sortedDoneRecordingList;
                lock (internal_vcrscheduler.Recordings.SyncRoot)
                {
                    // TODO: add ascending/descending setting + reimplement the sorting algorithm
                    sortedDoneRecordingList = Sorter.SortRecordingTable(internal_vcrscheduler.Recordings, true);
                }
                foreach (Recording recording_entry in sortedDoneRecordingList)
                {
                    Output.Append("<tr>            <td style=\"text-align: center; width: 60px;\">            <img src=\"images/waitbutton.png\" border=\"0\" /></td>            <td style=\"text-align: center; width: 169px;\">");

                    // OLD Output.Append("<tr style=\"background-color: #a29ca2;\"><td style=\"background-color:red;width:50px\"></td><td align=\"center\" style=\"width:150px\">");

                    // Start Date and Time
                    Output.Append(recording_entry.StartsAt.ToShortDateString());
                    Output.Append(" ");
                    Output.Append(recording_entry.StartsAt.ToShortTimeString());
                    Output.Append("</td><td style=\"text-align: center; width: 171px;\">");

                    // End Date and Time
                    Output.Append(recording_entry.EndsAt.ToShortDateString());
                    Output.Append(" ");
                    Output.Append(recording_entry.EndsAt.ToShortTimeString());

                    // Channel
                    Output.Append("</td><td style=\"text-align: center; width: 106px;\">  ");
                    // HACK: change that!!
                    // TODO: Channel Picture...
                    try
                    {
                        if (MapNames.HasPicture(Convert.ToInt32(recording_entry.Channel)))
                        {
                            Output.Append("<img src=\"");
                            Output.Append(MapNames.Name2Picture(MapNames.Number2Name(Convert.ToInt32(recording_entry.Channel))));
                            Output.Append("\" border=\"0\">");
                        }
                        else
                        {
                            Output.Append(recording_entry.Channel);
                        }
                    }
                    catch (Exception)
                    {
                        Output.Append(recording_entry.Channel);
                    }

                    // Recording Name
                    Output.Append("</td><td style=\"text-align: center; width: 277px;\">");
                    Output.Append(recording_entry.Recording_Name);
                    Output.Append("</td>");

                    // Categorie
                    // TODO: Add Categories...
                    Output.Append("<td style=\"text-align: center; width: 79px;\">todo</td>");
                    // Buttons
                    // TODO: Add Buttons...
                    Output.Append("<td style=\"text-align: center; width: 60px;\">                <img src=\"images/edit_button.png\" border=\"0\" alt=\"Editieren\" />");
                    Output.Append("<a href=\"/vcr/deleterecording?id=");
                    Output.Append(Convert.ToBase64String(recording_entry.Recording_ID.ToByteArray()));                        
                    Output.Append("\"><img src=\"images/delete_button.png\" border=\"0\" alt=\"Lschen\" /></A></TD><TR>");
                }
            }

            Output.Append("</table>");

            if ((internal_vcrscheduler.Recordings.Count == 0) && (internal_vcrscheduler.doneRecordings.Count == 0))
            {
                Output.Remove(0, Output.Length);
                Output.Append("No Recordings...");
            }

            return Output.ToString();
        }
        #endregion

        #region RecordedListing
        /// <summary>
        /// this generates the Record Listing HTML Sourcecode
        /// </summary>
        /// <returns>Record Listing Sourcecode</returns>
        String Template_Recorded_Listing()
        {
            StringBuilder Output = new StringBuilder();

            // start the table
            Output.Append("	<table id=\"TABLE2\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\">	<tr>		<td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 79px;\">            <strong>Gespielt</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); width: 169px; background-repeat: repeat-x; height: 21px; text-align: center\">            <strong>Aufgenommen am</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); width: 75px; background-repeat: repeat-x; height: 21px; text-align: center\">            <strong>Dauer</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 64px;\">            <strong>Sender</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 278px;\">            <strong>Name</strong></td>	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 129px;\">            <strong>Kategorie</strong></td>  	    <td style=\"background-image: url(images/Table_Headerbar_Tile.png); background-repeat: repeat-x; height: 21px; text-align: center; width: 79px;\">        </td>	</tr>	");
            // OLD Output.Append("<table width=\"90%\"><tr style=\"background-color: #93939a;\"><td align=\"center\">Time</td><td align=\"center\">Sender</td><td align=\"center\">Name</td></tr>");

            if (internal_vcrscheduler.doneRecordings.Count > 0)
            {
                List<Recording> sortedDoneRecordingList;
                lock (internal_vcrscheduler.doneRecordings.SyncRoot)
                {
                    // TODO: add ascending/descending setting + reimplement the sorting algorithm
                    sortedDoneRecordingList = Sorter.SortRecordingTable(internal_vcrscheduler.doneRecordings, true);
                }

                foreach (Recording recording_entry in sortedDoneRecordingList)
                {
                    // Playcount/NEW!
                    Output.Append("<tr><td style=\"text-align: center\">");
                    if (recording_entry.PlayCount == 0)
                        Output.Append("<img src=\"images/neu.png\" border=\"0\" />");
                    else
                        Output.Append(recording_entry.PlayCount);

                    Output.Append("</td>");

                    // OLD Output.Append("<tr style=\"background-color: #a29ca2;\"><td align=\"center\" style=\"width:150px\">");

                    // StartAt
                    Output.Append("<td style=\"text-align: center\">");
                    Output.Append(recording_entry.StartsAt.ToShortDateString());
                    Output.Append(" ");
                    Output.Append(recording_entry.StartsAt.ToShortTimeString());

                    Output.Append("</td>");

                    // Runtime in Minutes
                    TimeSpan runtimeticks = new TimeSpan((recording_entry.EndsAt.Ticks - recording_entry.StartsAt.Ticks));

                    Output.Append("<td style=\"text-align: center; width: 75px;\">");
                    Output.Append(Convert.ToString(runtimeticks.TotalMinutes));
                    Output.Append("mins</td>");

                    // Channel
                    Output.Append("<td style=\"text-align: center; width: 64px;\">");

                    // HACK: change that!!
                    // TODO: Channel Picture...
                    try
                    {
                        if (MapNames.HasPicture(Convert.ToInt32(recording_entry.Channel)))
                        {
                            Output.Append("<img src=\"");
                            Output.Append(MapNames.Name2Picture(MapNames.Number2Name(Convert.ToInt32(recording_entry.Channel))));
                            Output.Append("\" border=\"0\">");
                        }
                        else
                        {
                            Output.Append(recording_entry.Channel);
                        }
                    }
                    catch (Exception)
                    {
                        Output.Append(recording_entry.Channel);
                    }



                    // Recording Name
                    Output.Append("</td><td style=\"text-align: center; width: 278px;\">");
                    Output.Append(recording_entry.Recording_Name);
                    Output.Append("</td>");


                    // TODO: make it configurable...Internationalization
                    // TODO: make the servername configurable...

                    // Category
                    Output.Append("<td style=\"text-align: center; width: 129px;\">todo</td>");

                    // TODO: Add Buttons...
                    Output.Append("<td style=\"text-align: center; width: 60px;\">");

                    Output.Append("<a href=\"/");
                    Output.Append(recording_entry.Recording_ID.ToString());
                    Output.Append("\"><img src=\"images/watch_button.png\" border=\"0\" alt=\"Ansehen\" /></a>");

                    Output.Append("<img src=\"images/edit_button.png\" border=\"0\" alt=\"Editieren\" />");
                    Output.Append("<a href=\"/vcr/RemoveRecordingFile?id=");
                    Output.Append(recording_entry.Recording_ID.ToString());
                    Output.Append("\"><img src=\"images/delete_button.png\" border=\"0\" alt=\"Lschen\" /></A></TD><TR>");
                    
                }
            }

            Output.Append("</table>");

            if ((internal_vcrscheduler.Recordings.Count == 0) && (internal_vcrscheduler.doneRecordings.Count == 0))
            {
                Output.Remove(0, Output.Length);
                Output.Append("No Recordings...");
            }

            return Output.ToString();
        }
        #endregion

        #region ConsoleOutput
        /// <summary>
        /// this generates the Record Listing HTML Sourcecode
        /// </summary>
        /// <returns>Record Listing Sourcecode</returns>
        String Template_Console_Output()
        {
            StringBuilder Output = new StringBuilder();

            String[] ConsoleOutput = ConsoleOutputLogger.GetLoggedLines();

            Output.Append("<pre>");

            foreach (String line in ConsoleOutput)
            {
                Output.Append(line + "\n");
            }
            Output.Append("</pre>");
            return Output.ToString();
        }
        #endregion
        #endregion
    }
}
